@@ -22,6 +22,7 @@ from ethpm_types.abi import ABIType, MethodABI
 from packaging.version import Version
 
 from .client import BaseSafeClient, MockSafeClient, SafeClient, SafeTx, SafeTxConfirmation, SafeTxID
+from .config import SafeConfig
 from .exceptions import (
     ApeSafeError,
     NoLocalSigners,
@@ -32,6 +33,7 @@ from .exceptions import (
 )
 from .factory import SafeFactory
 from .packages import PackageType
+from .types import SafeCacheData
 from .utils import get_safe_tx_hash, order_by_signer
 
 if TYPE_CHECKING:
@@ -42,9 +44,23 @@ if TYPE_CHECKING:
 class SafeContainer(AccountContainerAPI):
     _accounts: dict[str, "SafeAccount"] = {}
 
+    @property
+    def config(self) -> SafeConfig:
+        return cast(SafeConfig, self.config_manager["safe"])
+
     @property
     def _account_files(self) -> Iterator[Path]:
-        yield from self.data_folder.glob("*.json")
+        account_files = list(self.data_folder.glob("*.json"))
+
+        # NOTE: Make sure these Safes exist in our local cache
+        for required_safe, safe_cache_data in self.config.require.items():
+            # NOTE: If alias `required_safe` already exists, skip overwriting it
+            if required_safe not in map(lambda p: p.stem, account_files):
+                safe_cache_file = self.data_folder / f"{required_safe}.json"
+                safe_cache_file.write_text(safe_cache_data.model_dump_json(), encoding="utf-8")
+                account_files.append(safe_cache_file)
+
+        yield from account_files
 
     @property
     def aliases(self) -> Iterator[str]:
@@ -153,27 +169,6 @@ class SafeContainer(AccountContainerAPI):
         """
         self._get_path(alias).unlink(missing_ok=True)
 
-    def create_client(self, key: str) -> BaseSafeClient:
-        if key in self.aliases:
-            safe = self.load_account(key)
-            return safe.client
-
-        elif key in self.addresses:
-            account = cast(SafeAccount, self[cast(AddressType, key)])
-            return account.client
-
-        elif key in self.aliases:
-            return self.load_account(key).client
-
-        else:
-            address = self.conversion_manager.convert(key, AddressType)
-            if address in self.addresses:
-                account = cast(SafeAccount, self[cast(AddressType, key)])
-                return account.client
-
-            # Is not locally managed.
-            return SafeClient(address=address, chain_id=self.chain_manager.provider.chain_id)
-
     def _get_path(self, alias: str) -> Path:
         return self.data_folder.joinpath(f"{alias}.json")
 
@@ -204,17 +199,21 @@ class SafeAccount(AccountAPI):
         return self.account_file_path.stem
 
     @property
-    def account_file(self) -> dict:
-        return json.loads(self.account_file_path.read_text())
+    def account_data(self) -> SafeCacheData:
+        return SafeCacheData.model_validate_json(self.account_file_path.read_text(encoding="utf-8"))
 
     @property
+    def deployed_chain_ids(self) -> list[int]:
+        return self.account_data.deployed_chain_ids
+
+    @cached_property
     def address(self) -> AddressType:
         try:
             ecosystem = self.provider.network.ecosystem
         except ProviderNotConnectedError:
             ecosystem = self.network_manager.ethereum
 
-        return ecosystem.decode_address(self.account_file["address"])
+        return ecosystem.decode_address(self.account_data.address)
 
     @cached_property
     def contract(self) -> "ContractInstance":
@@ -272,34 +271,32 @@ class SafeAccount(AccountAPI):
 
         return self.contract.setGuard(new_guard, **tx_args)
 
+    def get_client(
+        self, chain_id: Optional[int] = None, override_url: Optional[str] = None
+    ) -> BaseSafeClient:
+        if chain_id is None:
+            chain_id = self.provider.chain_id
+
+        if override_url is None:
+            env_override = os.environ.get("SAFE_TRANSACTION_SERVICE_URL")
+            if env_override:
+                override_url = env_override
+
+        if chain_id == 0 or (self.provider.network.is_local and self.provider.chain_id == chain_id):
+            return MockSafeClient(contract=self.contract)
+
+        return SafeClient(address=self.address, chain_id=chain_id, override_url=override_url)
+
     @cached_property
     def client(self) -> BaseSafeClient:
-        chain_id = self.provider.chain_id
-        override_url = os.environ.get("SAFE_TRANSACTION_SERVICE_URL")
-
-        if self.provider.network.is_local:
-            return MockSafeClient(contract=self.contract)
-
-        elif chain_id in self.account_file["deployed_chain_ids"]:
-            return SafeClient(
-                address=self.address, chain_id=self.provider.chain_id, override_url=override_url
-            )
-
-        elif (
+        if (
             self.provider.network.name.endswith("-fork")
             and isinstance(self.provider.network, ForkedNetworkAPI)
-            and self.provider.network.upstream_chain_id in self.account_file["deployed_chain_ids"]
+            and self.provider.network.upstream_chain_id in self.deployed_chain_ids
         ):
-            return SafeClient(
-                address=self.address,
-                chain_id=self.provider.network.upstream_chain_id,
-                override_url=override_url,
-            )
+            return self.get_client(chain_id=self.provider.network.upstream_chain_id)
 
-        elif self.provider.network.is_dev:
-            return MockSafeClient(contract=self.contract)
-
-        return SafeClient(address=self.address, chain_id=self.provider.chain_id)
+        return self.get_client()
 
     @property
     def version(self) -> Version:
@@ -428,6 +425,7 @@ class SafeAccount(AccountAPI):
 
         if submitter is not None and not isinstance(submitter, AccountAPI):
             submitter = self.load_submitter(submitter)
+            assert isinstance(submitter, AccountAPI)  # NOTE: mypy happy
 
         if (
             submitter is not None
@@ -685,6 +683,7 @@ class SafeAccount(AccountAPI):
 
         if not isinstance(submitter, AccountAPI):
             submitter = self.load_submitter(submitter)
+            assert isinstance(submitter, AccountAPI)  # NOTE: mypy happy
 
         return submitter.call(txn)
 
